package net.jaekl.cfb;

// Comparative FindBugs
// 
// Tool to compare successive runs of FindBugs, 
// flagging the change from one run to the next.
// 
// Copyright (C) 2015 Christian Jaekl

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;
import java.text.MessageFormat;
import java.util.Locale;

import net.jaekl.cfb.analyze.Analysis;
import net.jaekl.cfb.analyze.Analyzer;
import net.jaekl.cfb.db.CfbSchema;
import net.jaekl.cfb.db.driver.DbDriver;
import net.jaekl.cfb.db.driver.PostgresqlDriver;
import net.jaekl.qd.xml.XmlParseException;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

public class CFB {
	DbDriver m_driver;
	CfbSchema m_schema;
	CfbBundle m_bundle;	
	Locale m_locale;
	
	// Command-line parameters
	String m_dbName; // db name
	File m_fbp;		// FindBugsProject file
	File m_fbDir;	// Directory where FindBugs is installed
	String m_host;	// db host
	int m_port;		// db port
	String m_user;	// db user
	String m_pass;	// db password
	
	CFB(Locale locale) {
		m_driver = new PostgresqlDriver();
		m_schema = new CfbSchema(m_driver);
		m_locale = locale;
		m_bundle = CfbBundle.getInst(m_locale);
		
		m_dbName = "CFB";
		m_fbp    = null;
		m_fbDir  = null;
		m_host = "localhost";
		m_port = 5432;
		m_pass = "";
		m_user = "user";
	}
	
	Options createOptions() {
		Options opt = new Options();
		
		opt.addOption("d", "dbname", true, "DB name");
		opt.addOption("f", "fbp",    true, "FindBugsProject file");
		opt.addOption("h", "host",   true, "DB hostname");
		opt.addOption("p", "pass",   true, "DB password");
		opt.addOption("t", "port",   true, "DB port");
		opt.addOption("u", "user",   true, "DB username");
		
		return opt;
	}
	
	boolean parseArgs(PrintWriter pw, String[] args) {
		Options opt = createOptions();
		
		try {
			CommandLine line = new GnuParser().parse(opt, args);
			if (line.hasOption("d")) {
				m_dbName = line.getOptionValue("d");
			}
			if (line.hasOption("f")) {
				m_fbp = new File(line.getOptionValue("f"));
			}
			if (line.hasOption("h")) {
				m_host = line.getOptionValue("h");
			}
			if (line.hasOption("p")) {
				m_pass = line.getOptionValue("p");
			}
			if (line.hasOption("t")) {
				m_port = Integer.parseInt(line.getOptionValue("t"));
			}
			if (line.hasOption("u")) {
				m_user = line.getOptionValue("u");
			}
		} 
		catch (ParseException exc) {
			usage(pw, opt);
			return false;
		}
		
		return true;
	}
	
	void usage(PrintWriter pw, Options opt) {
		HelpFormatter help = new HelpFormatter();
		help.printHelp(pw, 80, getClass().getName(), "", opt, 0, 0, "", true);
	}
	
	String trans(String key) {
		return m_bundle.get(key);
	}
	
	String getenv(String varName) {
		// This is a separate function so that we can override it at unit test time
		return System.getenv(varName);
	}
	
	String getProperty(String propName) {
		// This is a separate function so that we can override it at unit test time
		return System.getProperty(propName);
	}
	
	void initArgs() {
		String findBugsDir = getenv("FINDBUGS_HOME");
		if (null != findBugsDir) {
			m_fbDir = new File(findBugsDir);
		}
		findBugsDir = getProperty("findbugs.home");
		if (null != findBugsDir) {
			m_fbDir = new File(findBugsDir);
		}
	} 
	
	void doMain(PrintWriter pw, String[] args) throws SQLException, IOException, XmlParseException {
		initArgs();	// read environment and system properties
		if ( ! parseArgs(pw, args) ) {
			return;
		}
		
		try (Connection con = m_driver.connect(m_host, m_port, m_dbName, m_user, m_pass)) {
			m_schema.ensureDbInitialized(con);			
		}
		catch (SQLException exc) {
			String cannotConnectFormat = trans(CfbBundle.CANNOT_CONNECT);
			String cannotConnect = MessageFormat.format(cannotConnectFormat, m_host, ""+m_port, m_dbName, m_user);
			exc.printStackTrace(pw);
			pw.println(cannotConnect);
			return;
		}
		
		File findBugsDir = (null != m_fbDir) ? m_fbDir : new File(".");
		File workDir = new File(".");
		Analyzer analyzer = new Analyzer(findBugsDir);
		Analysis analysis = analyzer.analyze(pw, workDir, m_fbp);
		if (null != analysis) {
			// TODO
		}
	}
	
	public static void main(String[] args) {
		CFB cfb = new CFB(Locale.getDefault());
		
		try (PrintWriter pw = new PrintWriter(System.out)){
			cfb.doMain(pw, args);
			pw.flush();
		} catch (SQLException | IOException | XmlParseException exc) {
			exc.printStackTrace();
		}
	}

}
